@@ -5,13 +5,13 @@
 """
 
 __author__ = 'Ryan Fisher <ryan.fisher@ligo.org>'
-__version__ = 'dqsegdb-release-1-0-2'
-__date__ = '2014-11-25 17:15:21 +0000'
+__version__ = 'ebe222'
+__date__ = '2015-04-30 20:57:55 +0000'
 
-version = 'dqsegdb-release-1-0-2'
-git_id = '09c3608144f76eb482e3a5754c30a28f48b94a9d'
-git_branch = None
-git_tag = 'dqsegdb-release-1-0-2'
+version = 'ebe222'
+git_id = 'ebe222716a7c488ced8611467b0952816558054b'
+git_branch = 'dqsegdb-1-1-0-branch'
+git_tag = None
 git_author = 'Ryan Fisher <ryan.fisher@ligo.org>'
 git_committer = 'Ryan Fisher <ryan.fisher@ligo.org>'
 git_status = 'UNCLEAN: Modified working tree'
